#include <iostream>
#include <math.h>
#include <fstream>
#include <algorithm>
#include <vector>

// For brevity...
using namespace std;

// The struct to store notes
struct cell_struct {
	int row, col, cell, size;
	bool notes[10];

	bool operator<(const cell_struct &b) const
	{
		return this->size < b.size;
	};
};

// The class to solve sudoku
class SudokuSolver
{
	public:
		SudokuSolver();

		void set_rows(int rows[][10]);
		void get_rows(int rows[][10]);

		bool load_file(string filename);
		bool write_file(string filename, bool print_notes = true);

		bool solve();

	private:
		void make_notes();

		bool check();

		int to_full(bool complete[], bool the_set[]);
		int join_sugg(bool joint[], bool a[], bool b[]);

		int get_macro(int row, int column);

		int rows[10][10];
		vector<cell_struct> notes;
};

// The constructor - initializes data
SudokuSolver::SudokuSolver()
{
	// Initialize variables
	int i, j;
	for(i = 1; i < 10; i++)
	{
		for(j = 1; j < 10; j++)
		{
			this->rows[i][j] = 0;
		}
	}
}

// Set rows in the instance
void SudokuSolver::set_rows(int rows[][10])
{
	int i, j;
	for(i = 1; i < 10; i++) {
		for(j = 1; j < 10; j++) {
			this->rows[i][j] = rows[i][j];
		}
	}
}

// Set the rows in the passed array
void SudokuSolver::get_rows(int rows[][10])
{
	int i, j;
	for(i = 1; i < 10; i++) {
		for(j = 1; j < 10; j++) {
			rows[i][j] = this->rows[i][j];
		}
	}
}

// Load the sudoku from a file
bool SudokuSolver::load_file(string filename)
{
	int num;
	int i, j;

	// Open the file
	ifstream in;
	in.open(filename.c_str());

	// Check the file
	if(!in.is_open()) {
		return false;
	}

	// Read the numbers
	for(i = 1; i < 10; i++) {
		for(j = 1; j < 10; j++) {
			in >> num;
			num = (num < 10 && num > 0) ? num : 0;
			this->rows[i][j] = num;
		}
	}

	// Close the file
	in.close();

	return true;
}

// Write the solved sudoku
bool SudokuSolver::write_file(string filename, bool print_notes)
{
	ofstream out;
	out.open(filename.c_str());

	// Check the file
	if(!out.is_open()) {
		return false;
	}

	// Write the numbers
	int i, j;
	for(i = 1; i < 10; i++) {
		for(j = 1; j < 10; j++) {
			out << this->rows[i][j] << " ";
		}
		out.seekp(-1, ios_base::cur);
		out << endl;
	}

	// Write notes
	if(print_notes) {
		out << endl << endl << "NOTES" << endl << endl;
		vector<cell_struct>::iterator it;
		for(it=this->notes.begin(); it!=this->notes.end(); ++it) {
			out << "Cell " << it->row << "x" << it->col << " " <<  it->size << ":";
			
			for(i = 1; i < 10; i++) {
				if(it->notes[i])
					out << " " << i;
			}
			out << endl;
		}
	}

	// Close the file
	out.close();

	return true;
}

// The resolver function
bool SudokuSolver::solve()
{
	// Assumed we've already data

	// The notes iterator
	vector<cell_struct>::iterator it;

	// Cells that have to be solved
	int to_solve = 90;

	// Resolve with notes
	bool changed = true;
	while(changed) {
		// Reset the change status
		changed = false;

		// Make notes again
		this->make_notes();

		// Sort notes and iterate them
		sort(this->notes.begin(), this->notes.end());
		for(it=this->notes.begin(); it!=this->notes.end(); ++it) {
			if(this->rows[it->row][it->col] != 0) {
			} else if(it->size == 1) {
				int i;
				for(i = 1; i < 10; i++) {
					if(it->notes[i]) {
						this->rows[it->row][it->col] = i;
						changed = true;
						break;
					}
				}
			}
		}

		// Set the number of cells to be solved
		to_solve = this->notes.size();
	}

	if(to_solve == 0)
		return this->check();

	// Brute force the sudoku
	SudokuSolver * bruteforcer = new SudokuSolver();
	int i;
	it = this->notes.begin();
	for(i = 1; i < 10; i++) {
		// Set the temporary value
		if(!it->notes[i])
			continue;
		this->rows[it->row][it->col] = i;

		// Brute force with that value
		bruteforcer->set_rows(this->rows);
		if(bruteforcer->solve()) {
			bruteforcer->get_rows(this->rows);
			return true;
		}

		// If the trial failed, set again 0
		this->rows[it->row][it->col] = 0;
	}

	return false;
}

// Make notes
void SudokuSolver::make_notes()
{
	int num;
	int i, j, k, size;

	bool row[10][10], col[10][10], cell[10][10], already[10];
	cell_struct the_cell;

	// Erase all previous notes
	this->notes.erase(this->notes.begin(), this->notes.end());

	// Initialize all to false
	for(i = 1; i < 10; i++) {
		for(j = 1; j < 10; j++) {
			row[i][j] = false;
			col[i][j] = false;
			cell[i][j] = false;
		}
	}

	// Store all inserted numbers
	for(i = 1; i < 10; i++) {
		for(j = 1; j < 10; j++) {
			num = this->rows[i][j];

			// If there isn't a number, ignore the cell
			if(num == 0)
				continue;

			row[i][num] = true;
			col[j][num] = true;
			cell[this->get_macro(i, j)][num] = true;
		}
	}

	// Make the notes with the set difference
	for(i = 1; i < 10; i++) {
		for(j = 1; j < 10; j++) {
			num = this->rows[i][j];

			// If there is a number, ignore the cell
			if(num != 0)
				continue;

			// Get the cell id
			k = this->get_macro(i, j);

			// Make the suggestions
			this->join_sugg(already, row[i], col[j]);
			this->join_sugg(already, already, cell[k]);
			size = this->to_full(the_cell.notes, already);

			// Save the suggestions
			the_cell.row = i;
			the_cell.col = j;
			the_cell.cell = k;
			the_cell.size = size;
			this->notes.push_back(the_cell);
		}
	}
}

// Check if the sudoku is correct
bool SudokuSolver::check()
{
	bool row[10][10], col[10][10];
	int i, j;

	// Initialize to false the arrays
	for(i = 1; i < 10; i++) {
		for(j = 0; j < 10; j++) {
			row[i][j] = false;
			col[i][j] = false;
		}
	}

	// Do the check
	for(i = 1; i < 10; i++) {
		for(j = 1; j <10; j++) {
			if(row[j][this->rows[i][j]] || col[i][this->rows[i][j]]) {
				return false;
			} else {
				row[j][this->rows[i][j]] = true;
				col[i][this->rows[i][j]] = true;
			}
		}
	}

	return true;
}

// Return the set to reach the full set
int SudokuSolver::to_full(bool complete[], bool the_set[])
{
	int size = 0;
	int i;
	for(i = 1; i < 10; i++) {
		complete[i] = !the_set[i];
		size += complete[i] ? 1 : 0;
	}

	return size;
}

// Join set of suggestions
int SudokuSolver::join_sugg(bool joint[], bool a[], bool b[])
{
	int ret = 0, i;
	for(i = 1; i < 10; i++) {
		joint[i] = a[i] || b[i];
		ret += joint[i] ? 1 : 0;
	}

	return ret;
}

// Return the macro cell from the row and the column
int SudokuSolver::get_macro(int row, int column)
{
	int urow, ucol;

	row--;
	column--;

	urow = floor(row / 3) * 3;
	ucol = floor(column / 3);

	return urow + ucol + 1;
}

// The main function
int main(int argc, char *argv[])
{
	SudokuSolver * s = new SudokuSolver();

	s->load_file("input.txt");

	bool ok = s->solve();
	if(ok)
		cout << "Sudoku solved :-)" << endl;
	else
		cout << "Sudoku not solved :-(" << endl;

	s->write_file("output.txt", !ok);
}